######################################################################
# --------------------------------------------------------------------
# Functions to search for changepoints in fractional polynomial models
# version from 1.9.2008
# --------------------------------------------------------------------
######################################################################
#
# Auxiliary functions:
# --------------------
# --------------------
# expit........... calculates expit(y)
# log.likelih6 ... calculates binomial likelihood for a model with 4 predictors (2 fractional polynomials of degree 2), data have to be ordered according to age!!!
# log.likelih7 ... calculates binomial likelihood for a model with 4 predictors, tapply have to be calculated in advance
# log.likelih8 ... calculates binomial likelihood for a model with 3 predictors, tapply have to be calculated in advance
# log.likelih9 ... calculates binomial likelihood for a model with 5 predictors, tapply have to be calculated in advance
#
# NO CHANGEPOINT
# ---------------------------------------------------
# ---------------------------------------------------
# b<-search.fracpoly.unc(data.be) ................. unconstrained powers, unconstrained coefficients
# b<-search.fracpoly.one(data.be) ................. degree one
# b<-search.fracpoly(data.be) ..................... constrained powers, unconstrained coefficients
#
# ONE CHANGEPOINT (<t, >=t)
# ---------------------------------------------------
# ---------------------------------------------------
#
# Changes only in the intercept
# -----------------------------
# b<-search.fracpoly.ch.2.intonly(data.be,theta.l,theta.u) ................. degree 2
# b<-search.fracpoly.ch.1.intonly(data.be,theta.l,theta.u) ................. degree 1
#
# Degree one models
# -----------------
# b<-search.fracpoly.ch.11.int(data.be,theta.l,theta.u) .............. also intercept change
# b<-search.fracpoly.ch.11(data.be,theta.l,theta.u) .................. no intercept change
#
# Higher degree models
# --------------------
# b<-search.fracpoly.ch.21.unc.int(data.be,theta.l,theta.u) .......... second-first model, also intercept change
# b<-search.fracpoly.ch.21.unc(data.be,theta.l,theta.u) .............. second-first model
# b<-search.fracpoly.ch.12.unc.int(data.be,theta.l,theta.u) .......... first-second model, also intercept change
# b<-search.fracpoly.ch.12.unc(data.be,theta.l,theta.u) .............. first-second model
# b<-search.fracpoly.ch.unc.int(data.be,theta.l,theta.u) ............. second order, also intercept change
# b<-search.fracpoly.ch.unc(data.be,theta.l,theta.u) ................. second order, no intercept change
#
# Models with constrained powers and unconstrained coefficients
# -------------------------------------------------------------
# b<-search.fracpoly.ch.21(data.be,theta.l,theta.u) ............. second-first order
# b<-search.fracpoly.ch(data.be,theta.l,theta.u) ................ second degree model
#
# Constrained models
# minimal AIC set to 3440
# -----------------------
# b<-search.fracpoly.ch.21.constr(data.be,theta.l,theta.u) .............. second-first model
# b<-search.fracpoly.ch.constr.int(data.be,theta.l,theta.u) ............. second order, also intercept change
# b<-search.fracpoly.ch.constr(data.be,theta.l,theta.u) ................. second order, no intercept change
# b<-search.fracpoly.ch.constr_second(data.be,theta.l,theta.u) .......... second order, first part unconstrained, second part constrained
#
# Models with forced continuity
# -----------------------------
# b<-search.fracpoly.ch.unc.cont(data.be,theta.l,theta.u) .............. second degree model
# b<-search.fracpoly.ch.12.cont(data.be,theta.l,theta.u) ............... first part degree one, second part degree two
# b<-search.fracpoly.ch.21.cont(data.be,theta.l,theta.u) ............... first part degree two, second part degree one
# b<-search.fracpoly.ch.one.cont(data.be,theta.l,theta.u) .............. one degree model
#
# Models proposed to the article (<=t, >t)
# ----------------------------------------
# b<-search.fracpoly.art(data.be,theta.l,theta.u) .............. degree one model
#
# TWO CHANGEPOINTS (<t, >=t&<=u, >u)
# ---------------------------------------------------
# ---------------------------------------------------
#
# All three parts different powers, all parts common intercept
# minimal AIC set to 3440
# ------------------------------------------------------------
# b<-search.fracpoly.ch2.one.3diff(data.be,theta.l,theta.u) .............. first degree model
# b<-search.fracpoly.ch2.211.3diff(data.be,theta.l,theta.u) .............. first part unconstrained degree two, the remaining parts degree one
# b<-search.fracpoly.ch2.3diff(data.be,theta.l,theta.u) .................. constrained powers degree two
# b<-search.fracpoly.ch2.3diff.unc.cons(data.be,theta.l,theta.u) ......... first part unconstrained, second and third part constrained powers degree two
# b<-search.fracpoly.ch2.3diff.unc(data.be,theta.l,theta.u) .............. unconstrained degree two model
#
# First and last part different powers, forced continuity between the first and second part
# minimal AIC set to 3440
# -----------------------------------------------------------------------------------------
# b<-search.fracpoly.ch2.one.3diff.cont(data.be,theta.l,theta.u) ........ first degree model
#
# First and last part the same powers, all parts common intercept
# minimal AIC set to 3440
# ---------------------------------------------------------------
# b<-search.fracpoly.ch2.one(data.be,theta.l,theta.u) .............. first degree model
# b<-search.fracpoly.ch2(data.be,theta.l,theta.u) .................. constrained powers degree two
# b<-search.fracpoly.ch2.unc(data.be,theta.l,theta.u) .............. unconstrained degree two model
#
# All parts the same powers, change only in the intercept of the middle part
# minimal AIC set to 3440
# --------------------------------------------------------------------------
# b<-search.fracpoly.ch2.one.intonly(data.be,theta.l,theta.u) .............. degree one
# b<-search.fracpoly.ch2.intonly(data.be,theta.l,theta.u) .................. degree two model, constrained powers
# b<-search.fracpoly.ch2.unc.intonly(data.be,theta.l,theta.u) .............. unconstrained degree two model
#
# -------------------------------------------------------------------------------------------------------------------------

########################################
# Reading and preparation of the data
#######################################

# data for Belgium
# ----------------
data<-read.table("parvovzvdata.txt",header=T)
names(data)[10]<-"age_midp_exact"
names(data)[12]<-"age_exact_cat"
names(data)[11]<-"sampl_year"

# preparation of the response variable
# ----------------------------------------------
vzvres.num<-rep(NA,length(data$vzvres))
vzvres.num[data$vzvres=="pos"]<-1
vzvres.num[data$vzvres=="neg"]<-0

parvores.num<-rep(NA,length(data$parvores))
parvores.num[data$parvores=="pos"]<-1
parvores.num[data$parvores=="neg"]<-0
# ----------------------------------------------

data<-data.frame(data,vzvres.num,parvores.num)

names(data)[3]<-"age_orig"
names(data)[10]<-"age"

data<-data[order(data$age),]

data.vzv<-data[!is.na(data$vzvres.num) & data$age<41 & data$age>0,] # # excluding the one patient older than 41 and one patient younger than zero ;)
# data.parvo<-data[!is.na(data$parvores.num) & data$age<72 & data$age>0.5,] # # excluding the three patients older than 72
data.parvo<-data[!is.na(data$parvores.num) & data$age<72 & data$age>0,] # # excluding the three patients older than 72
# to be able to use the functions written for the parvo data and to avoid the convergence problems
data.vzv.parvo<-data.frame(age=data.vzv$age+1, parvores.num=data.vzv$vzvres.num)
# to be consistent with the VZV analysis
data.parvo.2<-data.frame(age=data.parvo$age+1, parvores.num=data.parvo$parvores.num)

# Selection of the data
data.be<-data.parvo.2

#########################################################################################
# Auxiliary functions
#########################################################################################

# calculates binomial likelihood, ! data have to be ordered according to age !
# ----------------------------------------------------------------------------
log.likelih6 <- function(beta, data, term1, term2, term1.2, term2.2){
       ages<-(unique(data$age))
       pos <- tapply(data$parvores.num, data$age, sum)
       tot <- tapply(data$parvores.num, data$age, length)
       lin.pred<-(beta[1]+beta[2]*term1+beta[3]*term2+beta[4]*term1.2+beta[5]*term2.2)
          pi.a<-1 - 1/(1+exp(lin.pred))
          lik <- sum(pos*log(pi.a+3e-324)+(tot-pos)*log(1-pi.a+3e-324))
     return(-lik)
}

# calculates binomial likelihood for 4 predictors, tapply have to be calculated in advance
# ----------------------------------------------------------------------------------------
log.likelih7 <- function(beta, term1, term2, term1.2, term2.2, ages, pos, tot){
          lin.pred<-(beta[1]+beta[2]*term1+beta[3]*term2+beta[4]*term1.2+beta[5]*term2.2)
          pi.a<-1 - 1/(1+exp(lin.pred))
          lik <- sum(pos*log(pi.a+3e-324)+(tot-pos)*log(1-pi.a+3e-324))
     return(-lik)
}

# calculates binomial likelihood for 5 predictors, tapply have to be calculated in advance
# ----------------------------------------------------------------------------------------
log.likelih9 <- function(beta, term1, term2, term1.2, term2.2, term.int, ages, pos, tot){
          lin.pred<-(beta[1]+beta[2]*term1+beta[3]*term2+beta[4]*term1.2+beta[5]*term2.2+beta[6]*term.int)
          pi.a<-1 - 1/(1+exp(lin.pred))
          lik <- sum(pos*log(pi.a+3e-324)+(tot-pos)*log(1-pi.a+3e-324))
     return(-lik)
}

# calculates binomial likelihood for 3 predictors, tapply have to be calculated in advance
# ----------------------------------------------------------------------------------------
log.likelih8 <- function(beta, term1, term2, term1.2, ages, pos, tot){
          lin.pred<-(beta[1]+beta[2]*term1+beta[3]*term2+beta[4]*term1.2)
          pi.a<-1 - 1/(1+exp(lin.pred))
          lik <- sum(pos*log(pi.a+3e-324)+(tot-pos)*log(1-pi.a+3e-324))
     return(-lik)
}

# expit
# -----
expit<-function(y){return(1-1/(1+exp(y)))}

#########################################################################################
#########################################################################################
## NO CHANGEPOINT
#########################################################################################
#########################################################################################

########################################################
# unconstrained powers, unconstrained coefficients
# -----------------------------------------------------
search.fracpoly.unc<-function(data){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA)
     mistake<-c(NA, NA, NA)
     for (i in 1: (length(pow1))){
          for(j in i: (length(pow2))){
          if(pow1[i]==0){term1<-log(data$age)} else{term1<-(data$age)^(pow1[i])}
          if(pow2[j]==0){term2<-log(data$age)} else{term2<-(data$age)^(pow2[j])}
          if(pow1[i]==pow2[j]){term2<-log(data$age)*term1}
          glm.try<-glm(data$parvores.num~term1+term2, family="binomial")
          if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,i,j))}
          else{
               if(AIC(glm.try)<aic){
                    aic<-AIC(glm.try)
                    powers[1]<-pow1[i]
                    powers[2]<-pow2[j]
                                   }
               }
                                     }
                                 }
     return(list(powers=powers, aic=aic, mistake=mistake))
}

# b<-search.fracpoly.unc(data.be)

###########################
# polynomial of degree one
# ------------------------
search.fracpoly.one<-function(data){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     power<-NA
     mistake<-c(NA, NA)
     for (i in 1: (length(pow1))){
          if(pow1[i]==0){term1<-log(data$age)} else{term1<-(data$age)^(pow1[i])}
          glm.try<-glm(data$parvores.num~term1, family="binomial")
          if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,i))}
          else{
               if(AIC(glm.try)<aic){
                    aic<-AIC(glm.try)
                    power<-pow1[i]
                                   }
               }
                                     }
     return(list(power=power, aic=aic, mistake=mistake))
}

# b<-search.fracpoly.one(data.be)

#################################################
# constrained powers, unconstrained coefficients
# ----------------------------------------------
search.fracpoly<-function(data){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA)
     mistake<-c(NA, NA, NA)
     for (i in 1: (length(pow1))){
          glm.try<-glm(parvores.num~I(age^(pow1[i]))+I(log(age)), family="binomial", data=data)
          if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,i,0))}
          else{
               if(AIC(glm.try)<aic){
                    aic<-AIC(glm.try)
                    powers[1]<-pow1[i]
                    powers[2]<-0
                                   }
               }
          for(j in 2: (length(pow2))){
          glm.try<-glm(parvores.num~I(age^(pow1[i]))+I(age^(pow2[j])), family="binomial", data=data)
          if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,i,j))}
          else{
               if(AIC(glm.try)<aic){
                    aic<-AIC(glm.try)
                    powers[1]<-pow1[i]
                    powers[2]<-pow2[j]
                                   }
               }
                                     }
                                 }
     return(list(powers=powers, aic=aic, mistake=mistake))
}

# a<-search.fracpoly(data.be)

#################################################################################################
#################################################################################################
## ONE CHANGEPOINT
#################################################################################################
#################################################################################################

##########################################
# Changes only in the intercept
#########################################

###############################################################################
# two degree unconstrained fractional polynomial, change only in the intercept
# theta.l and theta.h should be integers
# --------------------------------------
search.fracpoly.ch.2.intonly<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA,NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA)
     age<-data$age
     len<-length(age)
     biclist<-c(NA,NA)
     for(tstar in (theta.l*2):(theta.h*2)){
          t<-tstar/2
          aict<-AIC(glm(parvores.num~age, family="binomial", data=data))
          powerst<-c(NA,NA)
          print(t)
          int2<-c(rep(1,len))
          int2[age<t]<-0
          for (i in 1: (length(pow1))){
               for(j in 1: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    glm.try<-glm(data$parvores.num~term1+term2+int2, family="binomial")
                    if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i))}
                                            else{
                                                if(AIC(glm.try)<aict){
                                                    aict<-AIC(glm.try)
                                                    powerst[1]<-pow1[i]
                                                    powerst[2]<-pow2[j]
                                                    bict<-AIC(glm.try,k=log(dim(data)[1]))
                                                    }
                                                }
                                   }
                                   }
                                   if(aict<aic){
                                   powers[1]<-powerst[1]
                                   powers[2]<-powerst[2]
                                   theta<-t
                                   }
                                   biclist=rbind(biclist,c(t,bict))
                                   }
     return(list(powers=powers, aic=aic, theta=theta, biclist=biclist,mistake=mistake))
}
a<-search.fracpoly.ch.2.intonly(data,range(data.be$age)[1],range(data.be$age)[2])

post.prob<-exp(-0.5*(a$biclist[-1,2]-min(a$biclist[-1,2])))/sum(exp(-0.5*(a$biclist[-1,2]-min(a$biclist[-1,2]))))
plot(a$biclist[-1,1],post.prob)
hist(rep(a$biclist[-1,1],post.prob*10e5),probability=T,breaks=seq(0.5,80.5,1),main="posterior density",xlab="theta")
plot(density(rep(a$biclist[-1,1],post.prob*10e5)),main="posterior density",xlab="theta")

#################################################################
# one degree fractional polynomial, change only in the intercept
# theta.l and theta.h should be integers
# --------------------------------------
search.fracpoly.ch.1.intonly<-function(data, theta.l, theta.h){
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-NA
     theta<-NA
     mistake<-c(NA,NA,NA)
     age<-data$age
     len<-length(age)
     biclist<-c(NA,NA)
          for(tstar in (theta.l):(theta.h)){
          t<-tstar
          aict<-AIC(glm(parvores.num~age, family="binomial", data=data))
          powerst<-NA
          print(t)
          int2<-c(rep(1,len))
          int2[age<t]<-0
          for (i in 1: (length(pow3))){
                    if(pow3[i]==0){term1<-log(age)} else{term1<-(age)^(pow3[i])}
                              glm.try<-glm(data$parvores.num~term1+int2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powerst<-pow3[i]
                                        bict<-AIC(glm.try,k=log(dim(data)[1]))
                                        theta<-t
                                                       }
                                   }
                                       }
                                   if(aict<aic){
                                   powers<-powerst
                                   theta<-t
                                   }
                                   biclist=rbind(biclist,c(t,bict))
                                   }
     return(list(powers=powers, aic=aic, theta=theta, biclist=biclist,mistake=mistake))
}
a<-search.fracpoly.ch.1.intonly(data,range(data.be$age)[1],range(data.be$age)[2])

# Best one for B19
post.prob<-exp(-0.5*(a$biclist[-1,2]-min(a$biclist[-1,2])))/sum(exp(-0.5*(a$biclist[-1,2]-min(a$biclist[-1,2]))))
plot(a$biclist[-1,1],post.prob)
hist(rep(a$biclist[-1,1],post.prob*10e5),probability=T,breaks=seq(0.5,80.5,1),main="posterior density",xlab="theta")
plot(density(rep(a$biclist[-1,1],post.prob*10e5)),main="posterior density",xlab="theta")

#######################################################
# One degree polynomials
######################################################

#########################################################
# one degree polynomials, a change also in the intercept
# ------------------------------------------------------
search.fracpoly.ch.11.int<-function(data, theta.l, theta.h){
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          int2<-c(rep(1, length(age)))
          int2[age<t]<-0
          for (i in 1: (length(pow3))){
                    if(pow3[i]==0){term1<-log(age)} else{term1<-(age)^(pow3[i])}
                    term1[age>=t]<-0
                         for (k in 1: (length(pow3))){
                              if(pow3[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow3[k])}
                              term1.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~int2+term1+term1.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,k))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow3[i]
                                        powers[2]<-pow3[k]
                                        theta<-t
                                                       }
                                   }
                                                       }
                                        }
                                   }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

##########################################
# one degree fractional polynomials
# theta.l and theta.h should be integers
# --------------------------------------
search.fracpoly.ch.11<-function(data, theta.l, theta.h){
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow3))){
                    if(pow3[i]==0){term1<-log(age)} else{term1<-(age)^(pow3[i])}
                    term1[age>=t]<-0
                         for (k in 1: (length(pow3))){
                              if(pow3[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow3[k])}
                              term1.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~term1+term1.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,k))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow3[i]
                                        powers[2]<-pow3[k]
                                        theta<-t
                                                       }
                                   }
                                                       }
                                        }
                                   }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#############################################
# Higher degrees fits
############################################

#########################################################################################
# first curve two degrees, second curve one degree, unconstrained, also intercept change
# theta.l and theta.h should be integers
# --------------------------------------
search.fracpoly.ch.21.unc.int<-function(data, theta.l, theta.h){
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
#    aic<-2887 # for poland
#     aic<-3440 # parvo be age midpoint exact
     powers<-c(NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          int2<-c(rep(1,length(age)))
          int2[age<t]<-0
          for (i in 1: (length(pow3))){
               for(j in i: (length(pow3))){
                    if(pow3[i]==0){term1<-log(age)} else{term1<-(age)^(pow3[i])}
                    if(pow3[j]==0){term2<-log(age)} else{term2<-(age)^(pow3[j])}
                    if(pow3[i]==pow3[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow3))){
                              if(pow3[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow3[k])}
                              term1.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~term1+term2+term1.2+int2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow3[i]
                                        powers[2]<-pow3[j]
                                        powers[3]<-pow3[k]
                                        theta<-t
                                                       }
                                   }
                                                    }
                                                     }
                                            }
                                        }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###################################################################
# first curve two degrees, second curve one degree, unconstrained
# theta.l and theta.h should be integers
# --------------------------------------
search.fracpoly.ch.21.unc<-function(data, theta.l, theta.h){
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow3))){
               for(j in i: (length(pow3))){
                    if(pow3[i]==0){term1<-log(age)} else{term1<-(age)^(pow3[i])}
                    if(pow3[j]==0){term2<-log(age)} else{term2<-(age)^(pow3[j])}
                    if(pow3[i]==pow3[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow3))){
                              if(pow3[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow3[k])}
                              term1.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~term1+term2+term1.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow3[i]
                                        powers[2]<-pow3[j]
                                        powers[3]<-pow3[k]
                                        theta<-t
                                                       }
                                   }
                                                    }
                                                     }
                                            }
                                        }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###########################################################################
# first curve degree one, second curve degree two, and an intercept change
# ------------------------------------------------------------------------
search.fracpoly.ch.12.unc.int<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
     print(t)
     int2<-c(rep(1, length(age)))
     int2[age<t]<-0
     for (i in 1: (length(pow1))){
          if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
          term1[age>=t]<-0
          for (k in 1: (length(pow1))){
               for(l in k: (length(pow1))){
                    if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                    if(pow1[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow1[l])}
                    if(pow1[k]==pow1[l]){term2.2<-log(age)*term1.2}
                    term1.2[age<t]<-0
                    term2.2[age<t]<-0
                    glm.try<-glm(data$parvores.num~int2+term1+term1.2+term2.2, family="binomial")
                    if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,k,l))}
                    else{
                         if(AIC(glm.try)<aic){
                              aic<-AIC(glm.try)
                              powers[1]<-pow1[i]
                              powers[3]<-pow1[k]
                              powers[4]<-pow1[l]
                              theta<-t
                                             }
                         }
                                          }
                                           }
                                   }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

##################################################
# first curve degree one, second curve degree two
# -----------------------------------------------
search.fracpoly.ch.12.unc<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
     print(t)
     for (i in 1: (length(pow1))){
          if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
          term1[age>=t]<-0
          for (k in 1: (length(pow1))){
               for(l in k: (length(pow1))){
                    if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                    if(pow1[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow1[l])}
                    if(pow1[k]==pow1[l]){term2.2<-log(age)*term1.2}
                    term1.2[age<t]<-0
                    term2.2[age<t]<-0
                    glm.try<-glm(data$parvores.num~term1+term1.2+term2.2, family="binomial")
                    if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,k,l))}
                    else{
                         if(AIC(glm.try)<aic){
                              aic<-AIC(glm.try)
                              powers[1]<-pow1[i]
                              powers[3]<-pow1[k]
                              powers[4]<-pow1[l]
                              theta<-t
                                             }
                         }
                                          }
                                           }
                                   }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

############################################################################
# unconstrained powers, unconstrained coefficients, and an intercept change
# -------------------------------------------------------------------------
search.fracpoly.ch.unc.int<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
     print(t)
     int2<-c(rep(1, length(age)))
     int2[age<t]<-0
          for (i in 1: (length(pow1))){
               for(j in i: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow1))){
                         for(l in k: (length(pow2))){
                              if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                              if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                              if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                              term1.2[age<t]<-0
                              term2.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~int2+term1+term2+term1.2+term2.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k,l))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow1[i]
                                        powers[2]<-pow2[j]
                                        powers[3]<-pow1[k]
                                        powers[4]<-pow2[l]
                                        theta<-t
                                                       }
                                   }
                                                    }
                                                     }
                                            }
                                        }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

####################################################
# unconstrained powers, unconstrained coefficients
# ------------------------------------------------
search.fracpoly.ch.unc<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
     print(t)
          for (i in 1: (length(pow1))){
               for(j in i: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow1))){
                         for(l in k: (length(pow2))){
                              if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                              if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                              if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                              term1.2[age<t]<-0
                              term2.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~term1+term2+term1.2+term2.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k,l))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow1[i]
                                        powers[2]<-pow2[j]
                                        powers[3]<-pow1[k]
                                        powers[4]<-pow2[l]
                                        theta<-t
                                                       }
                                   }
                                                    }
                                                     }
                                            }
                                        }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

##############################################################
# Constrained powers, unconstrained coefficients fits
##############################################################

###################################################
# first curve two degrees, second curve one degree
# theta.l and theta.h should be integers
# --------------------------------------
search.fracpoly.ch.21<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               for(j in 1: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow3))){
                              if(pow3[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow3[k])}
                              term1.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~term1+term2+term1.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow1[i]
                                        powers[2]<-pow2[j]
                                        powers[3]<-pow3[k]
                                        theta<-t
                                                       }
                                   }
                                                    }
                                                     }
                                            }
                                        }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#################################################
# constrained powers, unconstrained coefficients
# theta.l and theta.h should be integers
# --------------------------------------
search.fracpoly.ch<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               for(j in 1: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow1))){
                         for(l in 1: (length(pow2))){
                              if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                              if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                              if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                              term1.2[age<t]<-0
                              term2.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~term1+term2+term1.2+term2.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k,l))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow1[i]
                                        powers[2]<-pow2[j]
                                        powers[3]<-pow1[k]
                                        powers[4]<-pow2[l]
                                        theta<-t
                                                       }
                                   }
                                                    }
                                                     }
                                            }
                                        }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#############################################
# Constrained fits
#############################################

#####################################################
# first curve two degrees, second curve one degree
# theta.l and theta.h should be integers
# constrains the two-degree polynomial if necessary
# -------------------------------------------------
search.fracpoly.ch.21.constr<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     #aic<-3426.2 # AIC of the costrained version of the unconstrained model
     aic<-3440 # parvo be midpoint exact ages
     powers<-c(NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA)
     constr<-"no"
     age<-sort(data$age)
     parvores.num<-data$parvores.num[order(data$age)]
     Amat <- matrix(c(0,-1,0,0,0,0,1,0), nrow=2, ncol=4, byrow=T)
     age.un<-unique(age)
     pos <- tapply(parvores.num, age, sum)
     tot <- tapply(parvores.num, age, length)
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               for(j in 1: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow3))){
                              if(pow3[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow3[k])}
                              term1.2[age<t]<-0
                              glm.try<-glm(parvores.num~term1+term2+term1.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k))}
                              else{
                                   if(AIC(glm.try)<aic){
                                       cf<-coef(glm.try)
                                       if(cf[2]<0 & cf[3]>0){
                                            aic<-AIC(glm.try)
                                            powers[1]<-pow1[i]
                                            powers[2]<-pow2[j]
                                            powers[3]<-pow3[k]
                                            theta<-t
                                            constr<-"no"
                                                            }
                                        else{
                                             term1.u<-term1[!duplicated(age)]
                                             term2.u<-term2[!duplicated(age)]
                                             term1.2.u<-term1.2[!duplicated(age)]
                                             h<-constrOptim(theta = c(0.01, -0.01, 0.01, 0.01), f = log.likelih8, ui=Amat, ci=c(0,0), term1=term1.u, term2=term2.u, term1.2=term1.2.u, ages=age.un, tot=tot, pos=pos, method = "Nelder-Mead")
                                             if(h$convergence!=0){mistake<-rbind(mistake, c(2,t,i,j,k))}
                                             else{
                                                  h.aic<-2*(h$value)+8
                                                  if(h.aic<aic){
                                                       aic<-h.aic
                                                       powers[1]<-pow1[i]
                                                       powers[2]<-pow2[j]
                                                       powers[3]<-pow3[k]
                                                       theta<-t
                                                       constr<-"yes"
                                                                 }
                                                  }
                                              }
                                                       }
                                      }
                                            }
                                       }
                                       }
                                   }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake, constr=constr))
}

##########################################################################
# constrained powers, constrained coefficients, change in the intercept
# hlada changepoint v rozmedzi od theta.l do theta.h-teho veku v datasete
# vrati aj koeficienty, pri ktorych glm alebo constrOptim neskonvergovalo
# -----------------------------------------------------------------------
search.fracpoly.ch.constr.int<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     aic<-3440 # parvo be midpoint exact
     powers<-c(NA, NA, NA, NA)
     theta<-NA
     constr<-"no"
     age<-sort(data$age)
     parvores.num<-data$parvores.num[order(data$age)]
     mistake<-c(NA, NA, NA, NA, NA, NA)
     Amat <- matrix(c(0,-1,0,0,0,0,0,0,1,0,0,0,0,0,0,-1,0,0,0,0,0,0,1,0), nrow=4, ncol=6, byrow=T)
     age.un<-unique(age)
     pos <- tapply(parvores.num, age, sum)
     tot <- tapply(parvores.num, age, length)
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               int2<-c(rep(1,length(age)))
               int2[age<t]<-0
               for(j in 1: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow1))){
                              for(l in 1: (length(pow2))){
                                   if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                                   if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                                   if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                                   term1.2[age<t]<-0
                                   term2.2[age<t]<-0
                                   glm.try<-glm(parvores.num~term1+term2+term1.2+term2.2+int2, family="binomial")
                                   if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k,l))}
                                   else{
                                   if(AIC(glm.try)<aic){
                                       cf<-coef(glm.try)
                                       if(cf[2]<0 & cf[3]>0 & cf[4]<0 & cf[5]>0){
                                             aic<-AIC(glm.try)
                                             powers[1]<-pow1[i]
                                             powers[2]<-pow2[j]
                                             powers[3]<-pow1[k]
                                             powers[4]<-pow2[l]
                                             theta<-t
                                             constr<-"no"
                                                                                  }
                                        else{
                                             term1.u<-term1[!duplicated(age)]
                                             term2.u<-term2[!duplicated(age)]
                                             term1.2.u<-term1.2[!duplicated(age)]
                                             term2.2.u<-term2.2[!duplicated(age)]
                                             int2.u<-int2[!duplicated(age)]
                                             h<-constrOptim(theta = c(0.01, -0.01, 0.01, -0.01, 0.01, 0.01), f = log.likelih9, ui=Amat, ci=c(0,0,0,0), term1=term1.u, term2=term2.u, term1.2=term1.2.u, term2.2=term2.2.u, term.int=int2.u, ages=age.un, pos=pos, tot=tot, method = "Nelder-Mead")
                                             if(h$convergence!=0){mistake<-rbind(mistake, c(2,t,i,j,k,l))}
                                             else{
                                                  h.aic<-2*(h$value)+12
                                                  if(h.aic<aic){
                                                       aic<-h.aic
                                                       powers[1]<-pow1[i]
                                                       powers[2]<-pow2[j]
                                                       powers[3]<-pow1[k]
                                                       powers[4]<-pow2[l]
                                                       theta<-t
                                                       constr<-"yes"
                                                                 }
                                                  }
                                             }
                                                         }
                                             }
                                                         }
                                                         }
                                                       }
                                              }
                                          }
     return(list(powers=powers, aic=aic, theta=theta, constr=constr, mistake=mistake))
}

##########################################################################
# constrained powers, constrained coefficients
# hlada changepoint v rozmedzi od theta.l do theta.h-teho veku v datasete
# vrati aj koeficienty, pri ktorych glm alebo constrOptim neskonvergovalo
# -----------------------------------------------------------------------
search.fracpoly.ch.constr<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     aic<-3440 # parvo be midpoint exact
     powers<-c(NA, NA, NA, NA)
     theta<-NA
     constr<-"no"
     age<-sort(data$age)
     parvores.num<-data$parvores.num[order(data$age)]
     mistake<-c(NA, NA, NA, NA, NA, NA)
     Amat <- matrix(c(0,-1,0,0,0,0,0,1,0,0,0,0,0,-1,0,0,0,0,0,1), nrow=4, ncol=5, byrow=T)
     age.un<-unique(age)
     pos <- tapply(parvores.num, age, sum)
     tot <- tapply(parvores.num, age, length)
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               for(j in 1: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow1))){
                              for(l in 1: (length(pow2))){
                                   if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                                   if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                                   if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                                   term1.2[age<t]<-0
                                   term2.2[age<t]<-0
                                   glm.try<-glm(parvores.num~term1+term2+term1.2+term2.2, family="binomial")
                                   if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k,l))}
                                   else{
                                   if(AIC(glm.try)<aic){
                                       cf<-coef(glm.try)
                                       if(cf[2]<0 & cf[3]>0 & cf[4]<0 & cf[5]>0){
                                             aic<-AIC(glm.try)
                                             powers[1]<-pow1[i]
                                             powers[2]<-pow2[j]
                                             powers[3]<-pow1[k]
                                             powers[4]<-pow2[l]
                                             theta<-t
                                             constr<-"no"
                                                                                  }
                                        else{
                                             term1.u<-term1[!duplicated(age)]
                                             term2.u<-term2[!duplicated(age)]
                                             term1.2.u<-term1.2[!duplicated(age)]
                                             term2.2.u<-term2.2[!duplicated(age)]
                                             h  <-constrOptim(theta = c(0.01, -0.01, 0.01, -0.01, 0.01), f = log.likelih7, ui=Amat, ci=c(0,0,0,0), term1=term1.u, term2=term2.u, term1.2=term1.2.u, term2.2=term2.2.u, ages=age.un, pos=pos, tot=tot, method = "Nelder-Mead")
                                             if(h$convergence!=0){mistake<-rbind(mistake, c(2,t,i,j,k,l))}
                                             else{
                                                  h.aic<-2*(h$value)+10
                                                  if(h.aic<aic){
                                                       aic<-h.aic
                                                       powers[1]<-pow1[i]
                                                       powers[2]<-pow2[j]
                                                       powers[3]<-pow1[k]
                                                       powers[4]<-pow2[l]
                                                       theta<-t
                                                       constr<-"yes"
                                                                 }
                                                  }
                                             }
                                                         }
                                             }
                                                         }
                                                         }
                                                       }
                                              }
                                          }
     return(list(powers=powers, aic=aic, theta=theta, constr=constr, mistake=mistake))
}

###############################################################################################
# first part unconstrained, second part constrained powers&constrained coefficients degree two
# --------------------------------------------------------------------------------------------
search.fracpoly.ch.constr_second<-function(data, theta.l, theta.h){
     pow1c<-c(-2, -1, -0.5)
     pow2c<-c(0, 0.5, 1, 2, 3)
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440 # for be
     powers<-c(NA, NA, NA, NA)
     theta<-NA
     constr<-"no"
     age<-sort(data$age)
     parvores.num<-data$parvores.num[order(data$age)]
     mistake<-c(NA, NA, NA, NA, NA, NA)
     Amat <- matrix(c(0,0,0,-1,0,0,0,0,0,1), nrow=2, ncol=5, byrow=T)
     age.un<-unique(age)
     pos <- tapply(parvores.num, age, sum)
     tot <- tapply(parvores.num, age, length)
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               for(j in 1: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                         for (k in 1: (length(pow1c))){
                              for(l in 1: (length(pow2c))){
                                   if(pow1c[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1c[k])}
                                   if(pow2c[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2c[l])}
                                   if(pow1c[k]==pow2c[l]){term2.2<-log(age)*term1.2}
                                   term1.2[age<t]<-0
                                   term2.2[age<t]<-0
                                   glm.try<-glm(parvores.num~term1+term2+term1.2+term2.2, family="binomial")
                                   if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k,l))}
                                   else{
                                   if(AIC(glm.try)<aic){
                                       cf<-coef(glm.try)
                                       if(cf[4]<0 & cf[5]>0){
                                             aic<-AIC(glm.try)
                                             powers[1]<-pow1[i]
                                             powers[2]<-pow2[j]
                                             powers[3]<-pow1c[k]
                                             powers[4]<-pow2c[l]
                                             theta<-t
                                             constr<-"no"
                                                                                  }
                                        else{
                                             term1.u<-term1[!duplicated(age)]
                                             term2.u<-term2[!duplicated(age)]
                                             term1.2.u<-term1.2[!duplicated(age)]
                                             term2.2.u<-term2.2[!duplicated(age)]
                                             h<-constrOptim(theta = c(0.01, -0.01, 0.01, -0.01, 0.01), f = log.likelih7, ui=Amat, ci=c(0,0), term1=term1.u, term2=term2.u, term1.2=term1.2.u, term2.2=term2.2.u, ages=age.un, pos=pos, tot=tot, method = "Nelder-Mead")
                                             if(h$convergence!=0){mistake<-rbind(mistake, c(2,t,i,j,k,l))}
                                             else{
                                                  h.aic<-2*(h$value)+10
                                                  if(h.aic<aic){
                                                       aic<-h.aic
                                                       powers[1]<-pow1[i]
                                                       powers[2]<-pow2[j]
                                                       powers[3]<-pow1c[k]
                                                       powers[4]<-pow2c[l]
                                                       theta<-t
                                                       constr<-"yes"
                                                                 }
                                                  }
                                             }
                                                         }
                                             }
                                                         }
                                                         }
                                                       }
                                              }
                                          }
     return(list(powers=powers, aic=aic, theta=theta, constr=constr, mistake=mistake))
}

######################################
# Forced continuity
######################################

######################################################################
# unconstrained powers, unconstrained coefficients, forced continuity
# -------------------------------------------------------------------
search.fracpoly.ch.unc.cont<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               for(j in i: (length(pow2))){
                    if(pow1[i]==0){
                         term1<-log(age)
                         term1[age>=t]<-log(t)
                                   }
                    else{term1<-(age)^(pow1[i])
                         term1[age>=t]<-t^(pow1[i])
                         }
                    if(pow2[j]==0){
                         term2<-log(age)
                         term2[age>=t]<-log(t)
                                   }
                    else{term2<-(age)^(pow2[j])
                         term2[age>=t]<-t^(pow2[j])
                         }
                    if(pow1[i]==pow2[j]){
                         term2<-log(age)*term1
                         term2[age>=t]<-log(t)*term1[age>=t]
                                        }
                    for (k in 1: (length(pow1))){
                         for(l in k: (length(pow2))){
                              if(pow1[k]==0){term1.2<-log(age)-log(t)} else{term1.2<-(age)^(pow1[k])-(t)^(pow1[k])}
                              if(pow2[l]==0){term2.2<-log(age)-log(t)} else{term2.2<-(age)^(pow2[l])-t^(pow2[l])}
                              if(pow1[k]==pow2[l]){
                                   if(pow1[k]==0){term2.2<-(log(age))^2-(log(t))^2}
                                   else{term2.2<-log(age)*(age)^(pow1[k])-log(t)*(t)^(pow1[k])}
                                                  }
                              term1.2[age<t]<-0
                              term2.2[age<t]<-0
                              glm.try<-glm(data$parvores.num~term1+term2+term1.2+term2.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k,l))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow1[i]
                                        powers[2]<-pow2[j]
                                        powers[3]<-pow1[k]
                                        powers[4]<-pow2[l]
                                        theta<-t
                                                       }
                                   }
                                                    }
                                                     }
                                            }
                                        }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

########################################################################
# first part first degree, second part second degree, forced continuity
# ---------------------------------------------------------------------
search.fracpoly.ch.12.cont<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               if(pow1[i]==0){
                    term1<-log(age)
                    term1[age>=t]<-log(t)
                              }
               else{term1<-(age)^(pow1[i])
                    term1[age>=t]<-t^(pow1[i])
                    }
               for (k in 1: (length(pow1))){
                    for(l in k: (length(pow2))){
                         if(pow1[k]==0){term1.2<-log(age)-log(t)} else{term1.2<-(age)^(pow1[k])-(t)^(pow1[k])}
                         if(pow2[l]==0){term2.2<-log(age)-log(t)} else{term2.2<-(age)^(pow2[l])-t^(pow2[l])}
                         if(pow1[k]==pow2[l]){
                              if(pow1[k]==0){term2.2<-(log(age))^2-(log(t))^2}
                              else{term2.2<-log(age)*(age)^(pow1[k])-log(t)*(t)^(pow1[k])}
                                             }
                         term1.2[age<t]<-0
                         term2.2[age<t]<-0
                         glm.try<-glm(data$parvores.num~term1+term1.2+term2.2, family="binomial")
                         if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,k,l))}
                         else{
                              if(AIC(glm.try)<aic){
                                   aic<-AIC(glm.try)
                                   powers[1]<-pow1[i]
                                   powers[2]<-pow1[k]
                                   powers[3]<-pow2[l]
                                   theta<-t
                                                  }
                              }
                                               }
                                                }
                                        }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###################################################################
# first part degree two, second part degree one, forced continuity
# ----------------------------------------------------------------
search.fracpoly.ch.21.cont<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               for(j in i: (length(pow2))){
                    if(pow1[i]==0){
                         term1<-log(age)
                         term1[age>=t]<-log(t)
                                   }
                    else{term1<-(age)^(pow1[i])
                         term1[age>=t]<-t^(pow1[i])
                         }
                    if(pow2[j]==0){
                         term2<-log(age)
                         term2[age>=t]<-log(t)
                                   }
                    else{term2<-(age)^(pow2[j])
                         term2[age>=t]<-t^(pow2[j])
                         }
                    if(pow1[i]==pow2[j]){
                         term2<-log(age)*term1
                         term2[age>=t]<-log(t)*term1[age>=t]
                                        }
                    for (k in 1: (length(pow1))){
                         if(pow1[k]==0){term1.2<-log(age)-log(t)} else{term1.2<-(age)^(pow1[k])-(t)^(pow1[k])}
                         term1.2[age<t]<-0
                         glm.try<-glm(data$parvores.num~term1+term2+term1.2, family="binomial")
                         if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,j,k))}
                         else{
                              if(AIC(glm.try)<aic){
                                   aic<-AIC(glm.try)
                                   powers[1]<-pow1[i]
                                   powers[2]<-pow2[j]
                                   powers[3]<-pow1[k]
                                   theta<-t
                                                  }
                              }
                                                     }
                                            }
                                        }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###################################
# first degree, forced continuity
# -------------------------------
search.fracpoly.ch.one.cont<-function(data, theta.l, theta.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          for (i in 1: (length(pow1))){
               if(pow1[i]==0){
                    term1<-log(age)
                    term1[age>=t]<-log(t)
                              }
               else{term1<-(age)^(pow1[i])
                    term1[age>=t]<-t^(pow1[i])
                    }
               for (k in 1: (length(pow1))){
                    if(pow1[k]==0){term1.2<-log(age)-log(t)} else{term1.2<-(age)^(pow1[k])-(t)^(pow1[k])}
                    term1.2[age<t]<-0
                    glm.try<-glm(data$parvores.num~term1+term1.2, family="binomial")
                    if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,k))}
                    else{
                         if(AIC(glm.try)<aic){
                              aic<-AIC(glm.try)
                              powers[1]<-pow1[i]
                              powers[2]<-pow1[k]
                              theta<-t
                                                  }
                              }
                                                }
                                        }
                              }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#######################################################################
# a second type fracpoly to the article
#######################################################################
search.fracpoly.art<-function(data, theta.l, theta.h){
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-AIC(glm(parvores.num~age, family="binomial", data=data))
     powers<-c(NA, NA)
     theta<-NA
     mistake<-c(NA,NA,NA,NA)
     age<-data$age
     for(t in (theta.l):(theta.h)){
          print(t)
          int2<-c(rep(1, length(age)))
          int2[age<=t]<-0
          for (i in 1: (length(pow3))){
                    if(pow3[i]==0){term1<-log(age)} else{term1<-(age)^(pow3[i])}
                         for (k in (max(i,5)): (length(pow3))){
                              term1.2<-(age-t)^(pow3[k])
                              term1.2[age<=t]<-0
                              glm.try<-glm(data$parvores.num~int2+term1+term1.2, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,i,k))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow3[i]
                                        powers[2]<-pow3[k]
                                        theta<-t
                                                       }
                                   }
                                                       }
                                        }
                                   }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#######################################################################################################################
#######################################################################################################################
## TWO CHANGEPOINTS
#######################################################################################################################
#######################################################################################################################

###########################################################
# All parts different powers, all parts common intercept
##########################################################

##############
# degree one
# ----------
search.fracpoly.ch2.one.3diff<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440 
     powers<-c(NA, NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
               print(paste(t, u))
               for (i in 1: (length(pow1))){
                         if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                         term1[age>=t]<-0
                         for (k in 1: (length(pow1))){
                              if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                              term1.2[age<t|age>u]<-0
                              for (l in 1: (length(pow1))){
                                   if(pow1[l]==0){term1.3<-log(age)} else{term1.3<-(age)^(pow1[l])}
                                   term1.3[age<=u]<-0
                                   glm.try<-glm(data$parvores.num~term1+term1.2+term1.3, family="binomial")
                                   if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,k,l))}
                                   else{
                                        if(AIC(glm.try)<aic){
                                             aic<-AIC(glm.try)
                                             powers[1]<-pow1[i]
                                             powers[2]<-pow1[k]
                                             powers[3]<-pow1[l]
                                             theta[1]<-t
                                             theta[2]<-u
                                                            }
                                        }
                                                          }
                                                      }
                                               }
                                                  }
                                             }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

##################################################################
# first part unconstrained degree two, remaining parts degree one
# ---------------------------------------------------------------
search.fracpoly.ch2.211.3diff<-function(data, theta1.l, theta1.h){
     pow1<-pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440 
     powers<-c(NA, NA, NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
          print(paste(t, u))
          for (i in 1: (length(pow1))){
               for(j in 1: (length(pow2))){
                    if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                    if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                    if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                    term1[age>=t]<-0
                    term2[age>=t]<-0
                    for (k in 1: (length(pow1))){
                         if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                         term1.2[age<t|age>u]<-0
                         for (l in 1: (length(pow1))){
                              if(pow1[l]==0){term1.3<-log(age)} else{term1.3<-(age)^(pow1[l])}
                              term1.3[age<=u]<-0
                              glm.try<-glm(data$parvores.num~term1+term2+term1.2+term1.3, family="binomial")
                              if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,j,k,l))}
                              else{
                                   if(AIC(glm.try)<aic){
                                        aic<-AIC(glm.try)
                                        powers[1]<-pow1[i]
                                        powers[2]<-pow1[j]
                                        powers[3]<-pow1[k]
                                        powers[4]<-pow1[l]
                                        theta[1]<-t
                                        theta[2]<-u
                                                       }
                                   }
                                                     }
                                                 }
                                               }
                                                  }
                                             }
}
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#######################################################################
# degree two model with constrained powers, unconstrained coefficients
# --------------------------------------------------------------------
search.fracpoly.ch2.3diff<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     aic<-3440 
     powers<-c(NA, NA, NA, NA, NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
               print(paste(t,u))
               for (i in 1: (length(pow1))){
                    for(j in 1: (length(pow2))){
                         if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                         if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                         if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                         term1[age>=t]<-0
                         term2[age>=t]<-0
                              for (k in 1: (length(pow1))){
                              for(l in 1: (length(pow2))){
                                   if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                                   if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                                   if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                                   term1.2[age<t|age>u]<-0
                                   term2.2[age<t|age>u]<-0
                                   for (m in 1: (length(pow1))){
                                        for(n in 1: (length(pow2))){
                                             if(pow1[m]==0){term1.3<-log(age)} else{term1.3<-(age)^(pow1[m])}
                                             if(pow2[n]==0){term2.3<-log(age)} else{term2.3<-(age)^(pow2[n])}
                                             if(pow1[m]==pow2[n]){term2.3<-log(age)*term1.3}
                                             term1.3[age<=u]<-0
                                             term2.3[age<=u]<-0
                                             glm.try<-glm(data$parvores.num~term1+term2+term1.2+term2.2+term1.3+term2.3, family="binomial")
                                             if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,j,k,l,m,n))}
                                             else{
                                                  if(AIC(glm.try)<aic){
                                                       aic<-AIC(glm.try)
                                                       powers[1]<-pow1[i]
                                                       powers[2]<-pow2[j]
                                                       powers[3]<-pow1[k]
                                                       powers[4]<-pow2[l]
                                                       powers[5]<-pow1[m]
                                                       powers[6]<-pow2[n]
                                                       theta[1]<-t
                                                       theta[2]<-u
                                                                      }
                                                   }
                                                                    }
                                                                 }
                                                             }
                                                            }
                                                      }
                                             }
                                           }
                                        }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###########################################################################################################
# degree two, first term unconstrained, the remaining terms constrained powers, unconstrained coefficients
# --------------------------------------------------------------------------------------------------------
search.fracpoly.ch2.3diff.unc.cons<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     pow3<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440 
     powers<-c(NA, NA, NA, NA, NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
               print(paste(t,u))
               for (i in 1: (length(pow1))){
                    for(j in 1: (length(pow2))){
                         if(pow3[i]==0){term1<-log(age)} else{term1<-(age)^(pow3[i])}
                         if(pow3[j]==0){term2<-log(age)} else{term2<-(age)^(pow3[j])}
                         if(pow3[i]==pow3[j]){term2<-log(age)*term1}
                         term1[age>=t]<-0
                         term2[age>=t]<-0
                              for (k in 1: (length(pow1))){
                              for(l in 1: (length(pow2))){
                                   if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                                   if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                                   if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                                   term1.2[age<t|age>u]<-0
                                   term2.2[age<t|age>u]<-0
                                   for (m in 1: (length(pow1))){
                                        for(n in 1: (length(pow2))){
                                             if(pow1[m]==0){term1.3<-log(age)} else{term1.3<-(age)^(pow1[m])}
                                             if(pow2[n]==0){term2.3<-log(age)} else{term2.3<-(age)^(pow2[n])}
                                             if(pow1[m]==pow2[n]){term2.3<-log(age)*term1.3}
                                             term1.3[age<=u]<-0
                                             term2.3[age<=u]<-0
                                             glm.try<-glm(data$parvores.num~term1+term2+term1.2+term2.2+term1.3+term2.3, family="binomial")
                                             if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,j,k,l,m,n))}
                                             else{
                                                  if(AIC(glm.try)<aic){
                                                       aic<-AIC(glm.try)
                                                       powers[1]<-pow3[i]
                                                       powers[2]<-pow3[j]
                                                       powers[3]<-pow1[k]
                                                       powers[4]<-pow2[l]
                                                       powers[5]<-pow1[m]
                                                       powers[6]<-pow2[n]
                                                       theta[1]<-t
                                                       theta[2]<-u
                                                                      }
                                                   }
                                                                    }
                                                                 }
                                                             }
                                                            }
                                                      }
                                             }
                                           }
                                        }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###########################
# unconstrained degree two
# ------------------------
search.fracpoly.ch2.3diff.unc<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440 
     powers<-c(NA, NA, NA, NA, NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
               print(paste(t,u))
               for (i in 1: (length(pow1))){
                    for(j in 1: (length(pow2))){
                         if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                         if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                         if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                         term1[age>=t]<-0
                         term2[age>=t]<-0
                              for (k in 1: (length(pow1))){
                              for(l in 1: (length(pow2))){
                                   if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                                   if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                                   if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                                   term1.2[age<t|age>u]<-0
                                   term2.2[age<t|age>u]<-0
                                   for (m in 1: (length(pow1))){
                                        for(n in 1: (length(pow2))){
                                             if(pow1[m]==0){term1.3<-log(age)} else{term1.3<-(age)^(pow1[m])}
                                             if(pow2[n]==0){term2.3<-log(age)} else{term2.3<-(age)^(pow2[n])}
                                             if(pow1[m]==pow2[n]){term2.3<-log(age)*term1.3}
                                             term1.3[age<=u]<-0
                                             term2.3[age<=u]<-0
                                             glm.try<-glm(data$parvores.num~term1+term2+term1.2+term2.2+term1.3+term2.3, family="binomial")
                                             if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,j,k,l,m,n))}
                                             else{
                                                  if(AIC(glm.try)<aic){
                                                       aic<-AIC(glm.try)
                                                       powers[1]<-pow1[i]
                                                       powers[2]<-pow2[j]
                                                       powers[3]<-pow1[k]
                                                       powers[4]<-pow2[l]
                                                       powers[5]<-pow1[m]
                                                       powers[6]<-pow2[n]
                                                       theta[1]<-t
                                                       theta[2]<-u
                                                                      }
                                                   }
                                                                    }
                                                                 }
                                                             }
                                                            }
                                                      }
                                             }
                                           }
                                        }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#############################################################################
# All parts different powers, forced continuity between the first two parts
#############################################################################

##############
# degree one
# ----------
search.fracpoly.ch2.one.3diff.cont<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440
     powers<-c(NA, NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
               print(paste(t, u))
               for (i in 1: (length(pow1))){
                         if(pow1[i]==0){
                              term1<-log(age)
                              hlp<-log(t)}
                         else{
                              term1<-(age)^(pow1[i])
                              hlp<-t^pow1[i]}
                         term1[age>=t & age<=u]<-hlp
                         term1[age>u]<-0
                         for (k in 1: (length(pow1))){
                              if(pow1[k]==0){term1.2<-log(age)-log(t)} else{term1.2<-((age)^(pow1[k])-t^(pow1[k]))}
                              term1.2[age<t|age>u]<-0
                              for (l in 1: (length(pow1))){
                                   if(pow1[l]==0){term1.3<-log(age)} else{term1.3<-(age)^(pow1[l])}
                                   term1.3[age<=u]<-0
                                   glm.try<-glm(data$parvores.num~term1+term1.2+term1.3, family="binomial")
                                   if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,k,l))}
                                   else{
                                        if(AIC(glm.try)<aic){
                                             aic<-AIC(glm.try)
                                             powers[1]<-pow1[i]
                                             powers[2]<-pow1[k]
                                             powers[3]<-pow1[l]
                                             theta[1]<-t
                                             theta[2]<-u
                                                            }
                                        }
                                                          }
                                                      }
                                               }
                                                  }
                                             }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#######################################################################
# First and third part the same powers, all parts common intercept
#######################################################################

#############
# degree one
# ----------
search.fracpoly.ch2.one<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440
     powers<-c(NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
               print(paste(t, u))
               for (i in 1: (length(pow1))){
                         if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                         term1[age>=t&age<=u]<-0
                              for (k in 1: (length(pow1))){
                                   if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                                   term1.2[age<t|age>u]<-0
                                   glm.try<-glm(data$parvores.num~term1+term1.2, family="binomial")
                                   if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,k))}
                                   else{
                                        if(AIC(glm.try)<aic){
                                             aic<-AIC(glm.try)
                                             powers[1]<-pow1[i]
                                             powers[2]<-pow1[k]
                                             theta[1]<-t
                                             theta[2]<-u
                                                            }
                                        }
                                                       }
                                                       }
                                               }
                                             }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#############################################################
# degree two, constrained powers, unconstrained coefficients
# ----------------------------------------------------------
search.fracpoly.ch2<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     aic<-3440 
     powers<-c(NA, NA, NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
               print(paste(t,u))
               for (i in 1: (length(pow1))){
                    for(j in 1: (length(pow2))){
                         if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                         if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                         if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                         term1[age>=t&age<=u]<-0
                         term2[age>=t&age<=u]<-0
                              for (k in 1: (length(pow1))){
                                   for(l in 1: (length(pow2))){
                                        if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                                        if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                                        if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                                        term1.2[age<t|age>u]<-0
                                        term2.2[age<t|age>u]<-0
                                        glm.try<-glm(data$parvores.num~term1+term2+term1.2+term2.2, family="binomial")
                                        if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,j,k,l))}
                                        else{
                                             if(AIC(glm.try)<aic){
                                                  aic<-AIC(glm.try)
                                                  powers[1]<-pow1[i]
                                                  powers[2]<-pow2[j]
                                                  powers[3]<-pow1[k]
                                                  powers[4]<-pow2[l]
                                                  theta[1]<-t
                                                  theta[2]<-u
                                                       }
                                        }
                                                       }
                                                       }
                                               }
                                             }
                                           }
                                        }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###########################
# unconstrained degree two
# ------------------------
search.fracpoly.ch2.unc<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440 
     powers<-c(NA, NA, NA, NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA, NA, NA)
     age<-data$age
     for(t in (theta1.l):(theta1.h)){
          for(u in t:(trunc(max(data$age)))){
               print(paste(t,u))
               for (i in 1: (length(pow1))){
                    for(j in 1: (length(pow2))){
                         if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
                         if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
                         if(pow1[i]==pow2[j]){term2<-log(age)*term1}
                         term1[age>=t&age<=u]<-0
                         term2[age>=t&age<=u]<-0
                              for (k in 1: (length(pow1))){
                              for(l in 1: (length(pow2))){
                                   if(pow1[k]==0){term1.2<-log(age)} else{term1.2<-(age)^(pow1[k])}
                                   if(pow2[l]==0){term2.2<-log(age)} else{term2.2<-(age)^(pow2[l])}
                                   if(pow1[k]==pow2[l]){term2.2<-log(age)*term1.2}
                                   term1.2[age<t|age>u]<-0
                                   term2.2[age<t|age>u]<-0
                                   glm.try<-glm(data$parvores.num~term1+term2+term1.2+term2.2, family="binomial")
                                   if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,j,k,l))}
                                   else{
                                        if(AIC(glm.try)<aic){
                                             aic<-AIC(glm.try)
                                             powers[1]<-pow1[i]
                                             powers[2]<-pow2[j]
                                             powers[3]<-pow1[k]
                                             powers[4]<-pow2[l]
                                             theta[1]<-t
                                             theta[2]<-u
                                                       }
                                        }
                                                       }
                                                       }
                                               }
                                             }
                                           }
                                        }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###################################################################
# Only intercept change in the middle part
##################################################################

#############
# degree one
# ----------
search.fracpoly.ch2.one.intonly<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440 
     powers<-NA
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA)
     age<-data$age
     len<-length(age)
     for (i in 1: (length(pow1))){
          print(pow1[i])
          if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
          for(t in (theta1.l):(theta1.h)){
               for(u in t:(trunc(max(data$age)))){
                    int2<-c(rep(1,len))
                    int2[age<t|age>u]<-0
                    glm.try<-glm(data$parvores.num~term1+int2, family="binomial")
                    if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i))}
                    else{
                         if(AIC(glm.try)<aic){
                              aic<-AIC(glm.try)
                              powers<-pow1[i]
                              theta[1]<-t
                              theta[2]<-u
}
}
}
}
}
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

#############################################################
# degree two, constrained powers, unconstrained coefficients
# ----------------------------------------------------------
search.fracpoly.ch2.intonly<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5)
     pow2<-c(0, 0.5, 1, 2, 3)
     aic<-3440 # for be
     powers<-c(NA,NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA)
     age<-data$age
     len<-length(age)
     for (i in 1: (length(pow1))){
          for(j in 1: (length(pow2))){
               print(c(pow1[i], pow2[j]))
               if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
               if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
               if(pow1[i]==pow2[j]){term2<-log(age)*term1}
               for(t in (theta1.l):(theta1.h)){
                    for(u in t:(trunc(max(data$age)))){
                         int2<-c(rep(1,len))
                         int2[age<t|age>u]<-0
                         glm.try<-glm(data$parvores.num~term1+term2+int2, family="binomial")
                         if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,j))}
                         else{
                              if(AIC(glm.try)<aic){
                                   aic<-AIC(glm.try)
                                   powers[1]<-pow1[i]
                                   powers[2]<-pow2[j]
                                   theta[1]<-t
                                   theta[2]<-u
                                                  }
                              }
                                                         }
                                             }
                                    }
                               }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}

###########################
# unconstrained degree two
# ------------------------
search.fracpoly.ch2.unc.intonly<-function(data, theta1.l, theta1.h){
     pow1<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     pow2<-c(-2, -1, -0.5, 0, 0.5, 1, 2, 3)
     aic<-3440 # for be
     powers<-c(NA,NA)
     theta<-c(NA,NA)
     mistake<-c(NA, NA, NA, NA, NA)
     age<-data$age
     len<-length(age)
     for (i in 1: (length(pow1))){
          for(j in i: (length(pow2))){
               print(c(pow1[i], pow2[j]))
               if(pow1[i]==0){term1<-log(age)} else{term1<-(age)^(pow1[i])}
               if(pow2[j]==0){term2<-log(age)} else{term2<-(age)^(pow2[j])}
               if(pow1[i]==pow2[j]){term2<-log(age)*term1}
               for(t in (theta1.l):(theta1.h)){
                    for(u in t:(trunc(max(data$age)))){
                         int2<-c(rep(1,len))
                         int2[age<t|age>u]<-0
                         glm.try<-glm(data$parvores.num~term1+term2+int2, family="binomial")
                         if(glm.try$converged==FALSE){mistake<-rbind(mistake, c(1,t,u,i,j))}
                         else{
                              if(AIC(glm.try)<aic){
                                   aic<-AIC(glm.try)
                                   powers[1]<-pow1[i]
                                   powers[2]<-pow2[j]
                                   theta[1]<-t
                                   theta[2]<-u
                                                  }
                              }
                                                         }
                                             }
                                    }
                               }
     return(list(powers=powers, aic=aic, theta=theta, mistake=mistake))
}
